<?php
/*
Plugin Name: Simple File Downloader
Version: 1.0.0
Plugin URI: http://phplugins.softanalyzer.com/simple-file-downloader
Author: eugenealegiojo
Author URI: http://wpdevph.com
Description: Simplest way to add download links into your posts/pages.
*/

if ( !defined('SFD_URL') )
define( 'SFD_URL', plugins_url( basename( plugin_dir_path(__FILE__) ), basename( __FILE__ ) ) );

if ( !defined('SFD_PATH') )
define( 'SFD_PATH', plugin_dir_path( __FILE__ ) );
			
if ( !class_exists('SIMPLE_FILE_DOWNLOADER') ) {

	class SIMPLE_FILE_DOWNLOADER {
		
		public function __construct(){
			// TO DO: Add submenu for options
			//add_action('admin_menu', array(&$this, 'sdf_menu_options'));
			
			// Adding media button into the content editor
			add_action('media_buttons_context',  array(&$this, 'editor_download_button'), 10);
			
			// Adding popup window for file selection from media library
			add_action( 'admin_footer',  array(&$this, 'add_inline_popup_content') );
			
			// Adding admin footer scripts when download link is available'
			add_action('admin_enqueue_scripts', array(&$this, 'admin_footer_scripts'), 10 );
			
			// Do the ajax request for media files
			add_action( 'wp_ajax_get-media-location', array(&$this, 'get_media_files_attachment') );
			add_action( 'wp_ajax_nopriv_get-media-location', array(&$this, 'get_media_files_attachment') );
			
			// Shortcode support
			add_shortcode('media-downloader', array(&$this, 'sfd_media_downloader') ); 	
			
			// Process download
			add_action('init', array(&$this, 'sfd_process_download'), 10);
			
			// TO DO: Localization support
			//add_action('plugins_loaded', array(&$this,'sfd_plugins_loaded'), 10, 2 );
		}
		
		/**
		 * Admin footer scripts
		 */		
		function admin_footer_scripts( $hook ){
			if ( !in_array($hook, array('edit.php', 'post.php') ) ) return;
			
			wp_enqueue_script( 'sfd_admin_script', SFD_URL . '/js/admin.js' );
			wp_localize_script( 'sfd_admin_script', 'adminParam', array( 'ajaxURL' => admin_url('admin-ajax.php') ) );
		} 
		
		/**
		 * Localization handler
		 */
		function sfd_plugins_loaded(){
			load_plugin_textdomain( 'sfd', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
		}
		
		/**
		 * TO DO: Adding submenu in the media
		 */
		//function sdf_menu_options() {
			//add_submenu_page('upload.php', 'sdf-options', 'SFD Options', 'administrator', 'sdf-options-menu', array(&$this,'manage_sfd_options') );
		//}
		
		/**
		 * TO DO: File download options management
		 */
		//function manage_sfd_options(){
			
		//}

		/**
		 * Add custom media button for download file. This button will display popup that allows us to... 
		 * ...select file and generate shortcode to be inserted automatically in content editor.
		 *
		 * @return string $context
		 */	
		function editor_download_button(){
			//path to my icon
			$img = SFD_URL . '/images/download-16x16.png';

			//the id of the container I want to show in the popup
			$container_id = 'sfd_download_container';
		  
			//our popup's title
			$title = 'Add Downloader Link';

			//append the icon
			$context .= "<a class='thickbox button sfd_download_link' title='{$title}' href='#TB_inline?width=400&height=500&inlineId={$container_id}'>
						<img src='{$img}' / style='padding-bottom: 5px;'><span class='sfd_download_icon'>Add Download</span></a>";
		  
			return $context;
		}
		
		/**
		 * Add a popup content when download button is clicked.
		 *
		 * @return string $content
		 */
		function add_inline_popup_content() {
			$current_screen = get_current_screen();
			if( !in_array($current_screen->id, array('page', 'post')) ) return;
			
			$content = '<div id="sfd_download_container" style="display:none;">
					<div class="file-download-wrap">
							<div style="padding:15px 15px 0 15px;">
							<h2>'. __('Select file location', 'sfd') .'</h2></div>';
				// Select file location
				$content .= '<div style="padding: 15px 15px 0 15px">';
					$content .= '<span>'. __('Select a Media Library location from where you get the file.', 'sfd') .'</span><br />';
					$content .= '<select id="download_term_id"><option value="">-'. __('Select location', 'sfd') .'-</option>
									<option value="all-files">-'. __('All Media Library files', 'sfd') .'-</option>
									<option value="attached-files">-'. __('Attached to posts/pages', 'sfd') .'-</option>
								</select>';
				$content .= '</div>';
				
				// Select file to download
				$content .= '<div style="padding: 15px 15px 0 15px">
								<h2>'. __('Select media file', 'sfd') .'</h2>
								<span>'. __('Select a file from Media Library to add in the content as download link.', 'sfd') .'</span><br />'.
								__('<select id="download_attachment_id"><option value="">-No category has been selected', 'sfd') .'-</option></select>
							</div>';
							
				// Download texts			
				$content .= '<div style="padding: 15px 15px 0 15px">	
								<h2>'. __('Download texts', 'sfd') .'</h2>
								<span>'. __('Input texts you want to display for the link.', 'sfd') .'</span><br />
								<input type="text" name="downloader_texts" id="downloader_texts" size="40" />
								<br /><span><em>Default texts: "Download File"</em></span>
							</div>
							<div style="padding: 15px;";>
								<input type="button" onclick="sfd_InsertDownloadLink();" value="'. __('Generate Download Link', 'sfd'). '" id="insert-download-link" class="button-primary">
								<a onclick="tb_remove(); return false;" href="#" style="color:#bbb;" class="button">'. __('Cancel', 'sfd'). '</a>
							</div>
					</div></div>';
					
			echo $content;					
		}
		
		/***
		 * Load media files based from location selected (All media | Attached media)
		 * 
		 * @return string $return_data
		 */
		function get_media_files_attachment(){
			$media_location = $_POST['media_location'];
			if ( empty($media_location) ) return;
			
			$return_data = array();
			if( $media_location == 'attached-files' ) {
				$args = array('post_type' => 'attachment', 'posts_per_page' => -1, 'post_parent__not_in' => array(0), 'post_status' => 'inherit', 'orderby' => 'title', 'order' => 'ASC');
			} else {
				$args = array('post_type' => 'attachment', 'posts_per_page' => -1, 'post_parent' => 0, 'post_status' => 'inherit', 'orderby' => 'title', 'order' => 'ASC');
			}
			$get_media = get_posts( $args );
			if( $get_media ){
				foreach( $get_media as $key => $attachment ){
					$html .= '<option value="'. $attachment->ID .'">'. $attachment->post_title .'</option>';
				} // end foreach
				$return_data['message'] = $html;
				$return_data['error'] = 0;
			} else {
				$return_data['message'] = '<option value="">-'. __('No files found in this category.', 'sfd') .'-</option>';
				$return_data['error'] = 1;
			} // endif
			
			$response = json_encode($return_data);
			header('Content-Type: application/json');
			echo $response;
			die();
		} 
		
		/**
		 * Shortcode handler
		 *
		 * @param array $atts
		 * @return string $download_link
		 */
		function sfd_media_downloader( $atts ){
			global $post;
			
			extract( shortcode_atts( array(
				'media_id' => 0,
				'texts' => 'Download File',
				'image_url' => '',
				'class' => '',
				'size_texts' => '',
				'display_filesize' => ''
			), $atts ) );
			
			if ( $media_id > 0 ) {
				$link_class = (!empty($class)) ? ' class="'. $class .'"' : '';
				$texts = (!empty($image_url)) ? '<img src="'. $image_url .'" />' : $texts;
				$download_link = '<a href="'. get_bloginfo('wpurl') .'?media_dl='. $media_id .'"'.$link_class.'>'. $texts .'</a>';
				
				$get_media_item = wp_get_attachment_url( $media_id );
				$uploads = wp_upload_dir();
				$file_path = str_replace( $uploads['baseurl'], $uploads['basedir'], $get_media_item );
				$size_texts = (!empty($size_texts)) ? $size_texts : 'Size';
				if( !empty($display_filesize) && $display_filesize == 'yes' ) {
					$download_link .= ' '.strtoupper($size_texts) .': '. $this->formatSizeUnits(filesize($file_path));
				}
				return $download_link;
			}
		}
		
		/** 
		 * File downloader processor
		 * @return void
		 */
		function sfd_process_download(){
			if( isset($_GET['media_dl']) && !empty($_GET['media_dl']) ) {
				$media_file = $_GET['media_dl'];
				
				$media_id = $_GET['media_dl'];
				if ( (int)$media_id <= 0 ) return;
				
				// grab the requested file's name
				$file_name = get_attached_file( $media_id );
				
				// make sure it's a file before doing anything!
				if ( is_file($file_name) ) {
					
					// required for IE & Safari
					if(ini_get('zlib.output_compression')) { ini_set('zlib.output_compression', 'Off');	}
					
					// get the file mime type using the file extension
					switch(strtolower(substr(strrchr($file_name, '.'), 1))) {
						case 'pdf': $mime = 'application/pdf'; break;
						case 'zip': $mime = 'application/zip'; break;
						case 'jpeg':
						case 'jpg': $mime = 'image/jpg'; break;
						default: $mime = 'application/force-download';
					}
			
					header('Pragma: public'); 	// required
					header('Expires: 0');		// no cache
					header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
					header('Last-Modified: '.gmdate ('D, d M Y H:i:s', @filemtime ($file_name)).' GMT');
					header('Cache-Control: private',false);
					header('Content-Type: '.$mime);
					header('Content-Disposition: attachment; filename="'.basename($file_name).'"');
					header('Content-Transfer-Encoding: binary');
					header('Content-Length: '. @filesize($file_name) );	// provide file size
					header('Connection: close');
					readfile($file_name);		// push it out
					die();		
				}
			}	
		}
		
		/**
		 * Filesize formatting
		 * 
		 * @param int $bytes
		 * @return string $bytes
		 */
		function formatSizeUnits($bytes){
			if ($bytes >= 1073741824){
				 $bytes = number_format($bytes / 1073741824, 2) . ' GB';
			} elseif ($bytes >= 1048576) {
				 $bytes = number_format($bytes / 1048576, 2) . ' MB';
			} elseif ($bytes >= 1024) {
				$bytes = number_format($bytes / 1024, 2) . ' KB';
			} elseif ($bytes > 1) {
				$bytes = $bytes . ' bytes';
			} elseif ($bytes == 1) {
				$bytes = $bytes . ' byte';
			} else {
				$bytes = '0 bytes';
			}
			return $bytes;
		}
				
	}	// end class SIMPLE_FILE_DOWNLOADER
	
	$SimpleFileDownloader = new SIMPLE_FILE_DOWNLOADER();
}


